@@ -317,11 +317,6 @@ GLStateEGL::init_display(void* native_display, GLVisualConfig& visual_config)
         return false;
     }
 
-    if (gladLoadEGLUserPtr(EGL_NO_DISPLAY, load_egl_func, &egl_lib_) == 0) {
-        Log::error("Loading EGL entry points failed\n");
-        return false;
-    }
-
     native_display_ = reinterpret_cast<EGLNativeDisplayType>(native_display);
     requested_visual_config_ = visual_config;
 
@@ -341,21 +336,47 @@ GLStateEGL::init_gl_extensions()
 {
 #if GLMARK2_USE_GLESv2
     if (!gladLoadGLES2UserPtr(load_proc, this)) {
-        Log::error("Loading GLESv2 entry points failed.");
+        Log::error("Loading GLESv2 entry points failed.\n");
         return false;
     }
 
-    if (GLExtensions::support("GL_OES_mapbuffer")) {
-        GLExtensions::MapBuffer = glMapBufferOES;
-        GLExtensions::UnmapBuffer = glUnmapBufferOES;
-    }
+    GLExtensions::MapBuffer = glMapBufferOES;
+    GLExtensions::UnmapBuffer = glUnmapBufferOES;
+
+    GLExtensions::GenFramebuffers = glGenFramebuffers;
+    GLExtensions::DeleteFramebuffers = glDeleteFramebuffers;
+    GLExtensions::BindFramebuffer = glBindFramebuffer;
+    GLExtensions::FramebufferTexture2D = glFramebufferTexture2D;
+    GLExtensions::FramebufferRenderbuffer = glFramebufferRenderbuffer;
+    GLExtensions::CheckFramebufferStatus = glCheckFramebufferStatus;
+
+    GLExtensions::GenRenderbuffers = glGenRenderbuffers;
+    GLExtensions::DeleteRenderbuffers = glDeleteRenderbuffers;
+    GLExtensions::BindRenderbuffer = glBindRenderbuffer;
+    GLExtensions::RenderbufferStorage = glRenderbufferStorage;
+
+    GLExtensions::GenerateMipmap = glGenerateMipmap;
 #elif GLMARK2_USE_GL
     if (!gladLoadGLUserPtr(load_proc, this)) {
-        Log::error("Loading GL entry points failed.");
+        Log::error("Loading GL entry points failed.\n");
         return false;
     }
     GLExtensions::MapBuffer = glMapBuffer;
     GLExtensions::UnmapBuffer = glUnmapBuffer;
+
+    GLExtensions::GenFramebuffers = glGenFramebuffersEXT;
+    GLExtensions::DeleteFramebuffers = glDeleteFramebuffersEXT;
+    GLExtensions::BindFramebuffer = glBindFramebufferEXT;
+    GLExtensions::FramebufferTexture2D = glFramebufferTexture2DEXT;
+    GLExtensions::FramebufferRenderbuffer = glFramebufferRenderbufferEXT;
+    GLExtensions::CheckFramebufferStatus = glCheckFramebufferStatusEXT;
+
+    GLExtensions::GenRenderbuffers = glGenRenderbuffersEXT;
+    GLExtensions::DeleteRenderbuffers = glDeleteRenderbuffersEXT;
+    GLExtensions::BindRenderbuffer = glBindRenderbufferEXT;
+    GLExtensions::RenderbufferStorage = glRenderbufferStorageEXT;
+
+    GLExtensions::GenerateMipmap = glGenerateMipmapEXT;
 #endif
     return true;
 }
@@ -375,7 +396,7 @@ GLStateEGL::valid()
     if (!gotValidContext())
         return false;
 
-    if (egl_context_ == eglGetCurrentContext())
+    if (eglGetCurrentContext && egl_context_ == eglGetCurrentContext())
         return true;
 
     if (!eglMakeCurrent(egl_display_, egl_surface_, egl_surface_, egl_context_)) {
@@ -383,7 +404,8 @@ GLStateEGL::valid()
         return false;
     }
 
-    if (!eglSwapInterval(egl_display_, 0)) {
+    if (Options::swap_mode != Options::SwapModeFIFO &&
+        (!eglSwapInterval || !eglSwapInterval(egl_display_, 0))) {
         Log::info("** Failed to set swap interval. Results may be bounded above by refresh rate.\n");
     }
 
@@ -457,8 +479,6 @@ GLStateEGL::getVisualConfig(GLVisualConfig& vc)
 #define GLMARK2_NATIVE_EGL_DISPLAY_ENUM EGL_PLATFORM_WAYLAND_KHR
 #elif  GLMARK2_USE_DRM
 #define GLMARK2_NATIVE_EGL_DISPLAY_ENUM EGL_PLATFORM_GBM_KHR
-#elif  GLMARK2_USE_MIR
-#define GLMARK2_NATIVE_EGL_DISPLAY_ENUM EGL_PLATFORM_MIR_KHR
 #else
 // Platforms not in the above platform enums fall back to eglGetDisplay.
 #define GLMARK2_NATIVE_EGL_DISPLAY_ENUM 0
@@ -470,19 +490,37 @@ GLStateEGL::gotValidDisplay()
     if (egl_display_)
         return true;
 
+    /* Until we initialize glad EGL, load and use our own function pointers. */
+    PFNEGLQUERYSTRINGPROC egl_query_string =
+        reinterpret_cast<PFNEGLQUERYSTRINGPROC>(egl_lib_.load("eglQueryString"));
+    PFNEGLGETPROCADDRESSPROC egl_get_proc_address =
+        reinterpret_cast<PFNEGLGETPROCADDRESSPROC>(egl_lib_.load("eglGetProcAddress"));
+    PFNEGLGETERRORPROC egl_get_error =
+        reinterpret_cast<PFNEGLGETERRORPROC>(egl_lib_.load("eglGetError"));
+    PFNEGLGETDISPLAYPROC egl_get_display =
+        reinterpret_cast<PFNEGLGETDISPLAYPROC>(egl_lib_.load("eglGetDisplay"));
+    PFNEGLINITIALIZEPROC egl_initialize =
+        reinterpret_cast<PFNEGLINITIALIZEPROC>(egl_lib_.load("eglInitialize"));
+
+    if (!egl_query_string || !egl_get_proc_address || !egl_get_error ||
+        !egl_get_display || !egl_initialize)
+    {
+        return false;
+    }
+
     char const * __restrict const supported_extensions =
-        eglQueryString(EGL_NO_DISPLAY, EGL_EXTENSIONS);
+        egl_query_string(EGL_NO_DISPLAY, EGL_EXTENSIONS);
 
     if (GLMARK2_NATIVE_EGL_DISPLAY_ENUM != 0 && supported_extensions
         && strstr(supported_extensions, "EGL_EXT_platform_base"))
     {
         Log::debug("Using eglGetPlatformDisplayEXT()\n");
-        PFNEGLGETPLATFORMDISPLAYEXTPROC get_platform_display =
+        PFNEGLGETPLATFORMDISPLAYEXTPROC egl_get_platform_display =
             reinterpret_cast<PFNEGLGETPLATFORMDISPLAYEXTPROC>(
-                eglGetProcAddress("eglGetPlatformDisplayEXT"));
+                egl_get_proc_address("eglGetPlatformDisplayEXT"));
 
-        if (get_platform_display != nullptr) {
-            egl_display_ = get_platform_display(
+        if (egl_get_platform_display != nullptr) {
+            egl_display_ = egl_get_platform_display(
                 GLMARK2_NATIVE_EGL_DISPLAY_ENUM,
                 reinterpret_cast<void*>(native_display_),
                 nullptr);
@@ -490,7 +528,7 @@ GLStateEGL::gotValidDisplay()
 
         if (!egl_display_) {
             Log::debug("eglGetPlatformDisplayEXT() failed with error: 0x%x\n",
-                       eglGetError());
+                       egl_get_error());
         }
     }
     else
@@ -501,18 +539,18 @@ GLStateEGL::gotValidDisplay()
     /* Just in case get_platform_display failed... */
     if (!egl_display_) {
         Log::debug("Falling back to eglGetDisplay()\n");
-        egl_display_ = eglGetDisplay(native_display_);
+        egl_display_ = egl_get_display(native_display_);
     }
 
     if (!egl_display_) {
-        Log::error("eglGetDisplay() failed with error: 0x%x\n", eglGetError());
+        Log::error("eglGetDisplay() failed with error: 0x%x\n", egl_get_error());
         return false;
     }
 
     int egl_major(-1);
     int egl_minor(-1);
-    if (!eglInitialize(egl_display_, &egl_major, &egl_minor)) {
-        Log::error("eglInitialize() failed with error: 0x%x\n", eglGetError());
+    if (!egl_initialize(egl_display_, &egl_major, &egl_minor)) {
+        Log::error("eglInitialize() failed with error: 0x%x\n", egl_get_error());
         egl_display_ = 0;
         return false;
     }
@@ -523,6 +561,8 @@ GLStateEGL::gotValidDisplay()
         return false;
     }
 
+    /* From this point on we can use the normal EGL function calls. */
+
 #if GLMARK2_USE_GLESv2
     EGLenum apiType(EGL_OPENGL_ES_API);
 #if defined(WIN32)
@@ -533,9 +573,15 @@ GLStateEGL::gotValidDisplay()
 #elif GLMARK2_USE_GL
     EGLenum apiType(EGL_OPENGL_API);
     std::initializer_list<const char *> libNames = { "libGL.so", "libGL.so.1" };
+    if (!GLAD_EGL_VERSION_1_4) {
+        Log::error("EGL version %d.%d does not support the OpenGL API\n",
+                   egl_major, egl_minor);
+        return false;
+    }
 #endif
-    if (!eglBindAPI(apiType)) {
-        Log::error("Failed to bind api EGL_OPENGL_ES_API\n");
+
+    if (eglBindAPI && !eglBindAPI(apiType)) {
+        Log::error("Failed to bind api\n");
         return false;
     }
 
@@ -550,6 +596,7 @@ GLStateEGL::gotValidDisplay()
 void
 GLStateEGL::get_glvisualconfig(EGLConfig config, GLVisualConfig& visual_config)
 {
+    eglGetConfigAttrib(egl_display_, config, EGL_CONFIG_ID, &visual_config.id);
     eglGetConfigAttrib(egl_display_, config, EGL_BUFFER_SIZE, &visual_config.buffer);
     eglGetConfigAttrib(egl_display_, config, EGL_RED_SIZE, &visual_config.red);
     eglGetConfigAttrib(egl_display_, config, EGL_GREEN_SIZE, &visual_config.green);
@@ -557,6 +604,7 @@ GLStateEGL::get_glvisualconfig(EGLConfig config, GLVisualConfig& visual_config)
     eglGetConfigAttrib(egl_display_, config, EGL_ALPHA_SIZE, &visual_config.alpha);
     eglGetConfigAttrib(egl_display_, config, EGL_DEPTH_SIZE, &visual_config.depth);
     eglGetConfigAttrib(egl_display_, config, EGL_STENCIL_SIZE, &visual_config.stencil);
+    eglGetConfigAttrib(egl_display_, config, EGL_SAMPLES, &visual_config.samples);
 }
 
 EGLConfig
@@ -587,7 +635,7 @@ GLStateEGL::select_best_config(std::vector<EGLConfig>& configs)
         }
     }
 
-    return best_config;
+    return best_score > 0 ? best_config : 0;
 }
 
 bool
@@ -600,12 +648,6 @@ GLStateEGL::gotValidConfig()
         return false;
 
     const EGLint config_attribs[] = {
-        EGL_RED_SIZE, requested_visual_config_.red,
-        EGL_GREEN_SIZE, requested_visual_config_.green,
-        EGL_BLUE_SIZE, requested_visual_config_.blue,
-        EGL_ALPHA_SIZE, requested_visual_config_.alpha,
-        EGL_DEPTH_SIZE, requested_visual_config_.depth,
-        EGL_STENCIL_SIZE, requested_visual_config_.stencil,
 #if GLMARK2_USE_GLESv2
         EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
 #elif GLMARK2_USE_GL
@@ -639,6 +681,10 @@ GLStateEGL::gotValidConfig()
 
     // Select the best matching config
     egl_config_ = select_best_config(configs);
+    if (!egl_config_) {
+        Log::error("Failed to find suitable EGL config\n");
+        return false;
+    }
 
     vector<EglConfig> configVec;
     for (vector<EGLConfig>::const_iterator configIt = configs.begin();
